<?php
class qiwiapi {
    private $_phone;
    private $_token;
    private $_url;
    private $_form;
 
    function __construct($phone, $token) {
        $this->_phone = $phone;
        $this->_token = $token;
        $this->_url   = 'https://edge.qiwi.com/';
        $this->_form = 'https://qiwi.com/payment/form/';
    }
    private function sendRequest($method, $type = 'GET', array $content = []) {
        $ch = curl_init();
        if ($type == 'POST'){
            curl_setopt($ch, CURLOPT_URL, $this->_url . $method);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($content));
        }elseif($type == 'PUT'){
            curl_setopt($ch, CURLOPT_URL, $this->_url . $method .'/?'.http_build_query($content));
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'PUT');
        }elseif($type == 'DELETE'){
            curl_setopt($ch, CURLOPT_URL, $this->_url . $method .'/?'.http_build_query($content));
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'DELETE');
        }else{
            curl_setopt($ch, CURLOPT_URL, $this->_url . $method . '/?' . http_build_query($content));
        }
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Accept: application/json',
            'Content-Type: application/json',
            'Authorization: Bearer ' . $this->_token,
            'Host: edge.qiwi.com'
        ]);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $result = curl_exec($ch);
        curl_close($ch);
        return json_decode($result, 1);
    }
    public function doPay($id,array $params = []){
        if($this->checkHook() === true){
            return $this->getForm($id) . '?' . http_build_query($params);
        }
        return false;
    }
    public function getChechPay(){
        // Список параметров и подпись
        $Request = $this->getReqParams();
        // Вычисляем хэш SHA-256 строки параметров и шифруем с ключом для уведомлений
        $sign = $this->getSign($Request['payment']);
        // Проверка подписи запроса
        if (hash_equals($sign, $Request['hash'])) {
            return $Request['payment'];
        }else{
            return array('status' => 'error');
        }
    }
    private function getForm($id){
        return $this->_form . $id;
    }
    private function checkHook(){
        $hook = $this->getActiveHook();
        if(empty($hook['hookId'])){
            $hook = $this->CreateHook();
            if(empty($hook['hookId'])){
                return false;
            }
        }
        if(!empty($hook['hookId']) && $hook['hookParameters'] != $this->getCallbackUrl()){
            $delete = $this->deleteHook($hook['hookId']);
            if(!empty($delete['status']) && $delete['status'] == 'success'){
                $hook = $this->CreateHook();
                if(empty($hook['hookId'])){
                    return false;
                }
            }else{
                return false;
            }
        }
        return true;
    }
    private function getCallbackUrl(){
	return 'https://'.$_SERVER['HTTP_HOST'] . '/qiwi_merchant.php';
    }
    //Получение активного хука
    private function getActiveHook(){
        $response = $this->sendRequest('payment-notifier/v1/hooks/active');
        if (!empty($response['errorCode'])){
            return $this->getErrors($response);
        }else{
            return $response;
        }
    }
    //Создание хука
    private function CreateHook($txnType = '0'){
        $params = [
            'hookType' => 1,
            'param' => $this->getCallbackUrl(),
            'txnType' => $txnType
        ];
        $response = $this->sendRequest('payment-notifier/v1/hooks','PUT',$params);
        if (!empty($response['errorCode'])){
            return $this->getErrors($response);
        }
        return $response; 
    }
    //Удаление активного хука
    private function deleteHook($hook_id){
        $response = $this->sendRequest('payment-notifier/v1/hooks/'.$hook_id,'DELETE');
        if (!empty($response['errorCode'])){
            return $this->getErrors($response);
        }
        return array('status' => 'success', 'message' => 'Hook deleted.');
    }
    private function getSecretKey(){
        $hook = $this->getActiveHook();
        if (!empty($hook['hookId'])){
            $response = $this->sendRequest('payment-notifier/v1/hooks/'. $hook['hookId'] .'/key');
            if (!empty($response['errorCode'])){
                return $this->getErrors($response);
            }
            return $response['key'];
	}
        return;
    }
    //Функция возвращает упорядоченную строку значений параметров webhook и хэш подписи webhook для проверки
    private function getReqParams(){
        //Make sure that it is a POST request.
        if(strcasecmp($_SERVER['REQUEST_METHOD'], 'POST') != 0){
            throw new Exception('Request method must be POST!');
        }
        //Receive the RAW post data.
        $content = trim(file_get_contents("php://input"));
        //Attempt to decode the incoming RAW post data from JSON.
        $decoded = json_decode($content, true);
        //If json_decode failed, the JSON is invalid.
        if(!is_array($decoded)){
            throw new Exception('Received content contained invalid JSON!');
        }
        //Check if test
        if ($decoded['test'] == 'true') {
          throw new Exception('Test!');
        }
        return $decoded;
    }
    private function getSign($payment){
        $signParams = [
                $payment['sum']['currency'],
                $payment['sum']['amount'],
                $payment['type'],
                $payment['account'],
                $payment['txnId']
        ];
        // Вычисляем хэш SHA-256 строки параметров и шифруем с ключом для уведомлений
        return hash_hmac("sha256", implode('|', $signParams), base64_decode($this->getSecretKey()));
    }
    private function getErrors($response){
        return array('status' => 'error', 'errorCode' => $response['errorCode'],'message' => $response['description']);
    }
    public function getAccount(Array $params = []) {
        return $this->sendRequest('person-profile/v1/profile/current', $params);
    }
    public function getPaymentsHistory(Array $params = []) {
        return $this->sendRequest('payment-history/v2/persons/' . $this->_phone . '/payments', $params);
    }
    public function getPaymentsStats(Array $params = []) {
        return $this->sendRequest('payment-history/v2/persons/' . $this->_phone . '/payments/total', $params);
    }
    public function getTxn($txnId, Array $params = []) {
        return $this->sendRequest('payment-history/v2/transactions/' . $txnId .'/', $params);
    }
    public function getCheck($txnId, Array $params = []) {
	return $this->sendRequest('payment-history/v1/transactions/' . $txnId .'/cheque/file', $params);
    } 
    public function getBalance() {
        return $this->sendRequest('funding-sources/v2/persons/' . $this->_phone . '/accounts');
    }
    public function getTax($providerId) {
        return $this->sendRequest('sinap/providers/'. $providerId .'/form');
    } 
    public function sendMoneyToQiwi(Array $params = []) {
        return $this->sendRequest('sinap/api/v2/terms/99/payments', 'POST', $params);
    }
    public function sendMoneyToProvider($providerId, Array $params = []) {
        return $this->sendRequest('sinap/api/v2/terms/'. $providerId .'/payments', 'POST', $params);
    }
    public function sendMoneyToOther(Array $params = []) {
        return $this->sendRequest('sinap/api/v2/terms/1717/payments', 'POST', $params);
    }
}